function [] =CrockettExp(subID, intensity) % include intensity here for the shock 
                                     % If running without shocks, remove intensity here 
                                     % and comment out all code involving intensity

% Anahit Mkrtchian January 2015 - ICN, 17 Queen Square, London, UK. 

% This experiment runs a reinfored go no-go task based on Crockett et al.
% (2009) with a threat-of-shock paradigm. It runs 8 blocks in total, 
% alternating safe and threat conditions. 

% Participants are presented with checkerboards composed of blue and yellow
% tiles. They are asked to press spacebar if the target colour (blue or yellow - counterbalanced)
% is in the majority and to withold a keypress if the target colour is in
% the minority. 

% Experiment first runs a practice block of 48 trials presenting only the
% checkerboards. Then it runs 18 guided practice trials of each condition
% in blocks (4/condition). In the main task 36 trials of a neutral block
% (only checkerboards) are run, followed by the 4 experimental conditions (36 trials/condition).
% The experimental conditions are separated by a neutral block (36 trials). 

% Rewards:
    % small reward= 1 point + happy face
    % large reward = 10 points + happy face

% Punishments:
    % small punishment = lose 1 point + fearful face.
    % large punishment = lose 10 points + fearful face.


%  Experimental conditions:
    % 1. Reward-Go (Go to Win):
        % Correct Go = large reward.
        % Correct NoGo = small reward.
        % Incorrect response = no win/loss
    % 2. Reward-NoGo (NoGo to Win):
        % Correct NoGo= large reward
        % Correct Go= small reward
        % Incorrect response = no win/loss
    % 3. Punishment-Go (NoGo to Avoid losing):
        % Incorrect Go= large punishment
        % Incorrect NoGo= small punishment
        % Correct response = no win/loss
    % 4. Punishment-NoGo (Go to Avoid losing):
        % Incorrect NoGo= large punishment
        % Incorrect Go= small punishment
        % Correct response = no win/loss
       
% Calls functions:
    % KeyPressFunction.m - function for keypresses.
    % OJR_sendshock.m & JSW_SquareWaveShock.m - functions for sending
    % shock.
    % randCheckerboard.m - function for creating the checkerboards (stimuli)
        
% Image description:  
    % Happy and fearful faces were chosen from the Ekman set

    % Checkerboards: 
    % 5x5 checkerboards. 
    % Easy:ratio=16 blue or yellow tiles/9 blue or yellow tiles
    % Difficult: ratio = 13 blue or yellow tiles/12 blue or yellow tiles
    % All task conditions contain 50% go trials and 50% no-go trials, split evenly between easy and difficult stimuli  

% All trials are stored in the Trials folder    
% Experimental trials description:
    % Trials in each condition are stored in separate .csv files. There 2 files
    % per condition, one for each target colour (blue/yellow).
    % Reward Go: RewardGoBLUE.csv / RewardGoYELLOW.csv
    % Reward No Go: RewardNoGoBLUE.csv/ RewardGoBLUE.csv
    % Punishment Go: PunGoBLUE.csv / PunGoYELLOW.csv
    % Punishment No Go: PunNoGoBLUE.csv / PunNoGoYELLOW.csv

        % Column 1: type of checkerboard (1= easy, majority of tiles = YELLOW,2= easy, majority of tiles = BLUE, 3= difficult, majority of tiles = YELLOW, 4= difficult, majority of tiles = BLUE)
        % Column 2: type of outcome if pressed key (1= win 1 point, 10=win 10 points, -1=lose 1 point, -10=lose 10 points, 0=no win/loss)
        % Column 3: type of outcome if did not press key. 

    % neutral trials are stored in Neutral Block.csv
        % Column 1: checkerboard type
        
% Practice trials:
    % First block of practice trials are stored in PracticeBlock.csv
    % Guided practice trials are stored in
    % Practice(ConditionName)(targetcolour).csv files
    
% All instructions are stored in the Instructions folder in .txt files

% Data output file description:
% Results are stored in a text file for each partipant.

% Practice block:
    % Column 1: subID
    % Column 2: target colour (1-blue, 2- yellow)
    % Column 3: checkerboard type (1= easy, majority of tiles = YELLOW,2= easy, majority of tiles = BLUE, 3= difficult, majority of tiles = YELLOW, 4= difficult, majority of tiles = BLUE)
    % Column 4: If they made a correct (1) or incorrect (0) response.
    % Column 5: what response participant made (1-pressed key,
    % 0-no key press) 
    % Column 6: RT. If no keypress = stored as NaN.
    
% Experimental block:  
    % Column 1: subID
    % Column 2: Trial number
    % Column 3: type of trial (0-neutral block, 1- experimental condition)
    % Column 4: Threat (1) or safe (2) block
    % Column 5: target colour (1-blue, 2- yellow)
    % Column 6: checkerboard type 
    % Column 7: what experimental condition (1-reward go, 2-reward no go, 3- punishment go, 4- punishment no go)
    % Column 8: Type of coded outcome in trials file if pressed key (not for analysis)  
    % Column 9: Type of coded outcome in trials file if not pressed key (not for analysis) 
    % Column 10: Actual Outcome 
    % Column 11: what response participant made 
    % Column 12: If they made a correct (1) or incorrect (0) response (Accuracy)
    % Column 13: RT. If no keypress = stored as NaN.
    % Column 14: Mean RT from practice block. Set as the stim duration in
    % main task. 

%%

datestr(now) % date and time stamp. 

% Check if input is given:
if nargin < 2 % 
    error('Must provide required input parameter "subID" and "intensity".');
end

% Make sure shock intensity is not above 10. 
if intensity > 10
    error('Over 10. Please provide an intensity level below 10.');
end

% Set same keyboard mapping on all supported operating systems.
KbName('UnifyKeyNames');

% set the keys to use in the experiment.
space = KbName('space');
esc = KbName('ESCAPE');

% Define variables:
iti=0.25; % Intertrial interval=250ms
practiceDuration=2; % stimlulus presentation duration in the practice block=2000ms

% Set colour variables for stimuli:
blue=[0 255 225];
yellow=[225 225 0];

% Import practice trials: 48 trials.
practiceTrials=load('Trials/PracticeBlock.csv');
RandomPractice=practiceTrials(randperm(size(practiceTrials,1)),:); %randomise order

% Import neutral trial block: 36 trials.
neutralBlock=load('Trials/NeutralBlock.csv');
 
% THREAT/SAFE counterbalancing: swaps every other participant  
if mod(subID,2)==0
    TSblock_order = [1,2,1,2,1,2,1,2]; %threat is first
else
     TSblock_order = [2,1,2,1,2,1,2,1]; %safe is first
end

% Target colour counterbalancing: swaps every two participants.  
if mod(subID,4) == 1 || mod(subID,4) ==2 % if the participant ID is in one of two out of total 4 groups.     
    targetColour=1; % target colour is blue 
    text1 = fileread('Instructions/instructionsBLUE1.txt'); % import a text file with the instructions for when target colour is blue  
    text2=fileread('Instructions/instructionsBLUE2.txt');
    text3=fileread('Instructions/instructions3.txt');
    text4 = fileread('Instructions/instructions4.txt');
    text5 = fileread('Instructions/instructionsBLUE5.txt');
    textGuidedRGo = fileread('Instructions/guidedRgoBLUE.txt');
    textGuidedRNoGo = fileread('Instructions/guidedRNgoBLUE.txt');
    textGuidedPGo = fileread('Instructions/guidedPgoBLUE.txt');
    textGuidedPNoGo = fileread('Instructions/guidedPNgoBLUE.txt');
 
    % import the 4 guided practice conditions for when target colour is
    % blue:
    GuidedRGo=load('Trials/PracticeRGoBLUE.csv');
    GuidedRNoGo=load('Trials/PracticeRNoGoBLUE.csv');
    GuidedPGo=load('Trials/PracticePGoBLUE.csv');
    GuidedPNoGo=load('Trials/PracticePNoGoBLUE.csv');
    
    %import the 4 experimental conditions for when blue is the target
    %colour: 36 trials/condition.  
    RewardGoTrials=load('Trials/RewardGoBLUE.csv'); % load reward go blue trials. 
    RewardNoGoTrials=load('Trials/RewardNoGoBLUE.csv'); % load reward no go blue trials.
    PunGoTrials=load('Trials/PunGoBLUE.csv'); % load punishment go blue trials
    PunNoGoTrials=load('Trials/PunNoGoBLUE.csv'); % load punishment no go blue trials.
else  
    targetColour=2; % target colour is yellow
    text1 = fileread('Instructions/instructionsYELLOW1.txt'); % import a text file with the instructions for when target colour is yellow  
    text2=fileread('Instructions/instructionsYELLOW2.txt');
    text3=fileread('Instructions/instructions3.txt');
    text4 = fileread('Instructions/instructions4.txt');
    text5 = fileread('Instructions/instructionsYELLOW5.txt');
    textGuidedRGo = fileread('Instructions/guidedRgoYELLOW.txt');
    textGuidedRNoGo = fileread('Instructions/guidedRNgoYELLOW.txt');
    textGuidedPGo = fileread('Instructions/guidedPgoYELLOW.txt');
    textGuidedPNoGo = fileread('Instructions/guidedPNgoYELLOW.txt');
    
    % import the 4 guided practice conditions for when target colour is
    % yellow:
    GuidedRGo=load('Trials/PracticeRGoYELLOW.csv');
    GuidedRNoGo=load('Trials/PracticeRNoGoYELLOW.csv');
    GuidedPGo=load('Trials/PracticePGoYELLOW.csv');
    GuidedPNoGo=load('Trials/PracticePNoGoYELLOW.csv');
    
    %import the 4 experimental conditions for when yellow is the target
    %colour: 36 trials/condition.  
    RewardGoTrials=load('Trials/RewardGoYELLOW.csv'); % load reward go yellow trials. 
    RewardNoGoTrials=load('Trials/RewardNoGoYELLOW.csv'); % load reward no go yellow trials.
    PunGoTrials=load('Trials/PunGoYELLOW.csv'); % load punishment go yellow trials
    PunNoGoTrials=load('Trials/PunNoGoYELLOW.csv'); % load punishment no go yellow trials.
end  

% Female/male faces counterbalancing: swaps every four participants.
if mod(subID,8)<4 % use male face  
    [happy]=imread('CrockettStim/malehappy.bmp', 'bmp'); 
    [fear]=imread('CrockettStim/malefear.bmp', 'bmp'); 
else  % % use female face  
    [happy]=imread('CrockettStim/femalehappy.bmp', 'bmp'); 
    [fear]=imread('CrockettStim/femalefear.bmp', 'bmp');
end  

% experimental conditions
ExpTrials=cat(3,RewardGoTrials,RewardNoGoTrials,PunGoTrials,PunNoGoTrials); % create matrix with all conditions
sExpCond=randperm(4); % create a vector that randomises the 4 experimental conditions for the safe blocks: 1=rewardGo, 2=rewardNoGo, 3=PunGo, 4=PunNoGO. 
tExpCond=randperm(4); % create a vector that randomises the 4 experimental conditions for the threat blocks
ExpCond=[sExpCond(1),tExpCond(1), sExpCond(2), tExpCond(2), sExpCond(3),tExpCond(3), sExpCond(4), tExpCond(4)]; % create a vector that now has a randomised order of the experimental condition presentation. 

% Guided practice conditions
GuidedTrials=cat(3,GuidedRGo,GuidedRNoGo,GuidedPGo,GuidedPNoGo);
randomGuidedOrder=randperm(4); % create a vector that randomises the 4 guided conditions



%%
% =================
% file management
% ================

% Practice block 
dataPracticefilename = strcat('PracticeCrockett/LOGS_Crockett_',num2str(subID),'.dat'); % creates a filename based on participant number (subID)
datafilepointerPractice = fopen(dataPracticefilename,'w+'); % open new file overwriting any existing data = creates a datafile for each participant's practice block. 
fprintf(datafilepointerPractice,'%s %s %s %s %s %s %s \n','subID', 'target(1Blue2Yellow)', 'checkerboardType','Accuracy(1Correct)', 'response', 'rt', 'rtITI'); % write headings in columns 
fclose(datafilepointerPractice);  % close file in case anything gets written to it accidentally. 

% Experimental block
datafilename = strcat('dataCrockett/LOGS_Crockett_',num2str(subID),'.dat'); % creates a filename based on participant number (subID)
datafilepointer = fopen(datafilename,'w+'); % open new file overwriting any existing data = creates a datafile for each participant. 
fprintf(datafilepointer,'%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s \n','subID', 'TrialNo', 'neutral0expCond1','Threat(1)/Safe(2)', 'target(1Blue2Yellow)', 'checkerboardType', 'expCond(1RGo2RNoGo3PGo4PNoGo)', 'CodeOutcomePressed', 'CodeOutcomeNoPress','ActualOutcome', 'response','Accuracy(1Correct)', 'rt', 'rtITI', 'StimDuration'); % write headings in columns 
fclose(datafilepointer);  

%%  
% ===============
%   Experiment
% ===============
% code embedded in a 'try' and 'catch' statement. In case anything goes 
% wrong in the 'try' bit, the 'catch' bit is executed to make sure the 
% experiment will end and does not get stuck.

try
    % Set screen settings
    Screen('Preference', 'VisualDebuglevel', 3);   % replace the psychtoolbox startup screen with a black display. 
    %Screen('Preference', 'SkipSyncTests', 1);      % force Psychtoolbox to continue, despite the syncronization error ('PTB - ERROR: SYNCHRONIZATION FAILURE'). Only in Mac. Remember to remove this when actually testing. 
    screens=Screen('Screens');                     % Choosing the display with the highest display number to get the current display.
    screenNumber=max(screens);
    [w, wRect]=Screen('OpenWindow', screenNumber,[],[]); % Open window with default settings and get screen dimensions for text layout.
     % 200, 200, 800, 800 - use this when want to test it out, opens up a
     % smaller window
    HideCursor        

    % Set instruction page  
    Screen(w,'FillRect',[180 180 180]); 
    Screen('TextSize', w, 20);  
    Screen(w,'TextFont','Helvetica'); 
    Screen(w,'TextStyle',0);  
    DrawFormattedText(w, text1, 'center','center', [0 0 0],90);   
    Screen(w,'Flip');  
    [pressed, ~] = KeyPressFunction(space, inf); 
    
    
    DrawFormattedText(w, text2, 'center','center',  [0 0 0],90); 
    Screen(w,'Flip');  
    [pressed, RT] = KeyPressFunction(space, inf);

    %% Practice block
    if pressed==1 % Spacebar was pressed, start the practice block.          
        
        CorrectCounter=0; % counter for everytime a correct key response has been made. 
        CorrectRT=[]; % create an empty matrix and update it every time a correct key response has been made with the RTs for later averaging.
        for trialPracticeNo = 1:size(practiceTrials,1);
            PracticeTrial=RandomPractice(trialPracticeNo,:); 
            
            % ITI screen
            [X,Y] = RectCenter(wRect); 
            FixCross = [X-4,Y-30,X+4,Y+30;X-30,Y-4,X+30,Y+4]; 
            Screen('FillRect', w, [255,255,255], FixCross'); 
            Screen('Flip', w); 
            [pressedITI, RTiti] = KeyPressFunction([space,esc], iti);
            WaitSecs(iti-RTiti);
            if RTiti>=iti; 
                RTiti=NaN; 
            end
            if pressedITI ==2 
                break % 
            end
            
            %%% Stimuli presentation
            if PracticeTrial(1)==1;   % if stimulus is easy and majority of tiles are yellow.
                Screen('PutImage',w,randCheckerboard(blue,yellow,16,500,500)); 
            elseif PracticeTrial(1)==2;  % if stimulus is easy and majority of tiles are blue.   
                Screen('PutImage',w,randCheckerboard(yellow,blue,16,500,500));  
            elseif PracticeTrial(1)==3;  % if stimulus is difficult and majority of tiles are yellow.
                Screen('PutImage',w,randCheckerboard(blue,yellow,13,500,500));  
            else % if stimulus is difficult and majority of tiles are blue.
                Screen('PutImage',w, randCheckerboard(yellow,blue,13,500,500));    
            end
            
            Screen(w,'Flip'); 
            [pressed_lr, RT] = KeyPressFunction([space, esc], practiceDuration); 
            WaitSecs(practiceDuration-RT); 
            if RT>=practiceDuration; 
                RT=NaN; 
            end
            if pressed_lr ==2 
                break
            end
                
            % Accuracy recording:
            % participants were instructed to press a button (go) if the target color was 
            % in the majority and to not respond (no-go) if the target color was in the minority.
            GoNoGoResponse=0; % response is incorrect when it's 0 and correct when it is 1. 
            PressedCorrect=0;
            StimulusType=[1, 0, 1, 0]; % 1=majority is yellow, 0 = majority is blue. 
            if targetColour==1; % if target colour is blue  
                if StimulusType(PracticeTrial(1))==0 && pressed_lr==1; % if majority of tiles are blue and pressed a key
                        GoNoGoResponse=1; % store response as correct.
                        PressedCorrect=1; % stores a correct response for only pressed keys.
                elseif StimulusType(PracticeTrial(1))==1 && pressed_lr==0; % if majority of tiles are yellow and didn't press a key
                        GoNoGoResponse=1; % store response as correct.
                end
            else % target colour is yellow. 
               if StimulusType(PracticeTrial(1))==0 && pressed_lr==0; % if majority of tiles are blue and did not press a key...
                        GoNoGoResponse=1; % ...store response as correct.
                elseif StimulusType(PracticeTrial(1))==1 && pressed_lr==1; % if majority of tiles are yellow and did press a key...
                        GoNoGoResponse=1; % ...store response as correct.
                        PressedCorrect=1; % stores a correct response for only pressed keys.
               end  
            end
            
            
            if PressedCorrect==1; %if pressed a key and it was correct
                CorrectCounter=CorrectCounter+1; % update counter
                CorrectRT(CorrectCounter)=RT; % insert RT for correct key response in matrix 
            end
            
                    
            %%%% Save the responses to outputfile
            datafilepointerPractice = fopen(dataPracticefilename,'a+');
            fprintf(datafilepointerPractice,'%d %d %d %d %d %d %d \n',subID, targetColour, PracticeTrial(1), GoNoGoResponse, pressed_lr, RT,RTiti); 
            fclose(datafilepointerPractice);  
          
        end
        
        % Calculate mean RT: 
        if CorrectCounter>0; %if participant has at least one correct key press
            StimDuration=mean(CorrectRT); % set stimulus duration (in experimental block) to the mean of the correct key presses
        else % if participant does not have any correct key presses...
            StimDuration=2; %...set stimulus duration time to 2000ms.
        end
        
        %% Guided Practice trials
        
        DrawFormattedText(w, text3, 'center','center',  [0 0 0],90); 
        Screen(w,'Flip');  
        [pressed, RT] = KeyPressFunction(space, inf);

        if pressed==1 
            for guidedOrder=randomGuidedOrder;
                CurrentGuidedCond=GuidedTrials(:,:,guidedOrder);

                Screen('TextSize', w, 20); 
                Screen(w,'TextFont','Helvetica'); 
                Screen(w,'TextStyle',0);                

                if guidedOrder==1;
                    DrawFormattedText(w, textGuidedRGo, 'center','center', [0 0 0],90); 
                elseif guidedOrder==2;
                    DrawFormattedText(w, textGuidedRNoGo, 'center','center', [0 0 0],90); 
                elseif guidedOrder==3;
                    DrawFormattedText(w, textGuidedPGo, 'center','center', [0 0 0],90); 
                else
                    DrawFormattedText(w, textGuidedPNoGo, 'center','center', [0 0 0],90); 
                end

                Screen(w,'Flip'); 
                [pressed, ~] = KeyPressFunction(space, inf); 

                if pressed ==1;
                    for GuidedTrial=1:size(CurrentGuidedCond,1);
                        guidedTrialNo=CurrentGuidedCond(GuidedTrial,:);

                        % ITI screen
                        [X,Y] = RectCenter(wRect); 
                        FixCross = [X-4,Y-30,X+4,Y+30;X-30,Y-4,X+30,Y+4]; 
                        Screen('FillRect', w, [255,255,255], FixCross'); 
                        Screen('Flip', w); 
                        [pressed, ~] = KeyPressFunction(esc, iti);
                        if pressed ==1 
                            break 
                        end

                        %%% Stimuli presentation
                        if guidedTrialNo(1)==1;  
                            Screen('PutImage',w,randCheckerboard(blue,yellow,16,500,500)); 
                        elseif guidedTrialNo(1)==2;     
                            Screen('PutImage',w,randCheckerboard(yellow,blue,16,500,500));
                        elseif guidedTrialNo(1)==3; 
                            Screen('PutImage',w,randCheckerboard(blue,yellow,13,500,500)); 
                        else 
                            Screen('PutImage',w, randCheckerboard(yellow,blue,13,500,500));    
                        end

                        Screen(w,'Flip'); 
                        [pressed_lr, RT] = KeyPressFunction([space, esc], practiceDuration); 
                         WaitSecs(practiceDuration-RT);
                        if pressed_lr ==2 
                            break 
                        end

                        % ITI screen
                        [X,Y] = RectCenter(wRect); 
                        FixCross = [X-4,Y-30,X+4,Y+30;X-30,Y-4,X+30,Y+4]; 
                        Screen('FillRect', w, [255,255,255], FixCross'); 
                        Screen('Flip', w); 
                        [pressed, ~] = KeyPressFunction(esc, iti);
                        if pressed ==1                             
                            break 
                        end

                        %%%% Outcome
                        % First check how the participant responded. Then see
                        % what the outcome should be (reward, punishment, nothing).
                        if pressed_lr == 0; % no go - did not respond
                            outcome = guidedTrialNo(3); % the outcome is preprogrammed in the third element of the trial 
                        else  
                            outcome = guidedTrialNo(2); % the outcome is preprogrammed in the second element of the trial
                        end

                        % then place the outcome images on the screen based on the reward
                        % participants can receive 10, 1 points or lose 10, 1 points.
                        Screen('TextSize', w, 35); 
                        if outcome ==10;  % large reward: 10 points and a happy face
                            Screen('PutImage',w, happy);
                            DrawFormattedText(w, '+10 POINTS!', 'center',550, [0 0 0],90);  
                        elseif outcome == 1; % small reward: 1 point and a happy face 
                            Screen('PutImage',w, happy);
                            DrawFormattedText(w, '+1 POINT!', 'center',550, [0 0 0],90); 
                        elseif outcome== -10 % large punishment: lost 10 points and a fearful face
                            Screen('PutImage',w, fear);
                            DrawFormattedText(w, '-10 POINTS!', 'center',550, [0 0 0],90); 
                        elseif outcome==-1 % small punishment:lost 1 point and a fearful face
                            Screen('PutImage',w, fear);
                            DrawFormattedText(w, '-1 POINT!', 'center',550, [0 0 0],90); 
                        end 

                        Screen(w,'Flip');
                        [pressed, ~] = KeyPressFunction(esc, 1); % time allowed =  1000ms
                        if pressed ==1 
                            break 
                        end   
                    end
                end
            end
        end
            
        %% Experimental block
        % Set instruction page for experimental block 
        Screen(w,'FillRect',[180 180 180]); 
        Screen('TextSize', w, 20);  
        DrawFormattedText(w, text4, 'center','center',  [0 0 0],90);  
        Screen(w,'Flip');
        [pressed2, RT] = KeyPressFunction([space, esc], inf); 
        
        DrawFormattedText(w, text5, 'center','center',  [0 0 0],90);  
        Screen(w,'Flip'); 
        [pressed2, RT] = KeyPressFunction([space, esc], inf); 
            
        if pressed2==1  % Spacebar was pressed, do the experiment. 
            debrief=1; % debrief will allow to terminate the experiment and  whenever the participant wants to by pressing ESCAPE.  
            threatBlockCount=0; % threat block counter.
            TSBlockCounter=1; % threat and safe block counter.  
                
            for TScondition = TSblock_order; % repeat for each condition threat/ safe (order is defined near the top)
                if debrief==0 % if ESCAPE is pressed...  
                    break % ...break out of this for loop and go to debrief page. 
                elseif TScondition == 1  % threat condition
                    threatBlockCount=threatBlockCount+1; % keeps track of how many threat conditions you've been in.
                    Screen(w,'FillRect',[200 0 0]); % set background colour to red.
                    Screen('TextSize', w, 30); 
                    Screen(w,'TextStyle',1);  
                    DrawFormattedText(w, '!WARNING!', 'center',100, [255 255 255],90);   
                    DrawFormattedText(w, 'YOU ARE NOW AT RISK OF SHOCK', 'center','center', [255 255 255],90); 
                    Screen(w,'Flip');   
                    [pressed, ~] = KeyPressFunction(esc, 4); 
                    if pressed ==1 
                        break 
                    end
                else       % safe condition
                    Screen(w,'FillRect',[0 0 200]); 
                    Screen('TextSize', w, 30); 
                    Screen(w,'TextStyle',1);  
                    DrawFormattedText(w, 'YOU ARE NOW COMPLETELY SAFE', 'center','center', [255 255 255],90);  
                    Screen(w,'Flip'); 
                    [pressed, ~] = KeyPressFunction(esc, 4);
                    if pressed ==1     
                        break 
                    end
                end
                
                %% Neutral trials
                %Randomise the neutral trial order:
                RandomNeutral=neutralBlock(randperm(size(neutralBlock,1)),:);  
              
                % loop through the neutral trials first
                for neutralTrialNo= 1:size(neutralBlock,1);  
                    NeutralTrials=RandomNeutral(neutralTrialNo,:);   
               
                    % ITI screen
                    [X,Y] = RectCenter(wRect); 
                    FixCross = [X-4,Y-30,X+4,Y+30;X-30,Y-4,X+30,Y+4]; 
                    Screen('FillRect', w, [255,255,255], FixCross');  
                    Screen('Flip', w); 
                    [pressedITI, RTiti] = KeyPressFunction([space,esc], iti);
                    WaitSecs(iti-RTiti);
                    if RTiti>=iti; 
                        RTiti=NaN; 
                    end
                    if pressedITI ==2 
                        debrief=0;
                        break 
                    end
                        
                    %%% Stimuli presentation
                    if NeutralTrials(1)==1;  % if stimulus is easy and majority of tiles are yellow.
                        Screen('PutImage',w,randCheckerboard(blue,yellow,16,500,500)); 
                    elseif NeutralTrials(1)==2;  % if stimulus is easy and majority of tiles are blue.   
                        Screen('PutImage',w,randCheckerboard(yellow,blue,16,500,500)); 
                    elseif NeutralTrials(1)==3; % if stimulus is difficult and majority of tiles are yellow.
                        Screen('PutImage',w,randCheckerboard(blue,yellow,13,500,500));  
                    else % if stimulus is difficult and majority of tiles are blue.
                        Screen('PutImage',w, randCheckerboard(yellow,blue,13,500,500)); 
                    end
                    
                    Screen(w,'Flip'); 
                    [pressed_lr, RT] = KeyPressFunction([space, esc], StimDuration);  
                    WaitSecs(StimDuration-RT); 
                    if RT>=StimDuration; 
                        RT=NaN; 
                    end
                    if pressed_lr ==2 
                        debrief=0;
                        break 
                    end

                    % Accuracy recording
                    GoNoGoResponse=0; % response is incorrect when it's 0 and correct when it is 1. 
                    StimulusType=[1, 0, 1, 0]; % 1=majority is yellow, 0 = majority is blue. 
                    if targetColour==1; % if target colour is blue  
                        if StimulusType(NeutralTrials(1))==0 && pressed_lr==1; % if majority of tiles are blue and pressed a key
                                GoNoGoResponse=1; % store response as correct.
                        elseif StimulusType(NeutralTrials(1))==1 && pressed_lr==0; % if majority of tiles are yellow and didn't press a key
                                GoNoGoResponse=1; % store response as correct.
                        end
                    else % target colour is yellow. 
                       if StimulusType(NeutralTrials(1))==0 && pressed_lr==0; % if majority of tiles are blue and did not press a key...
                                GoNoGoResponse=1; % ...store response as correct.
                        elseif StimulusType(NeutralTrials(1))==1 && pressed_lr==1; % if majority of tiles are yellow and did press a key...
                                GoNoGoResponse=1; % ...store response as correct.
                       end  
                    end
                    
                    %%%% Save the responses to outputfile
                    datafilepointer = fopen(datafilename,'a+');
                    fprintf(datafilepointer,'%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d \n',subID, neutralTrialNo, 0, TScondition, targetColour, NeutralTrials(1), nan, nan, nan, nan, pressed_lr, GoNoGoResponse, RT, RTiti, StimDuration); 
                    fclose(datafilepointer);
                end
           
    
                %% Experimental conditions
                    
                CurrentExpCond=ExpTrials(:,:,ExpCond(TSBlockCounter)); % Picks out what experimental condition to present. save ExpCond(TSBlockCounter) to output file = give you the experimental condition: 1 = RewardGo,2 = RewardNoGo, 3=PunGo, 4=PunNoGo.
                RandomTrials=CurrentExpCond(randperm(size(CurrentExpCond,1)),:); % randomise the trial order of the current experimental condition. 
                
                Screen('TextSize', w, 20);  
                Screen(w,'TextFont','Helvetica'); 
                Screen(w,'TextStyle',1);                
    
                % Presents an instruction screen with the type of exp
                % condition that will follow
                if ExpCond(TSBlockCounter)==1;
                    DrawFormattedText(w, textGuidedRGo, 'center','center', [255,255,255],90); 
                elseif ExpCond(TSBlockCounter)==2;
                    DrawFormattedText(w, textGuidedRNoGo, 'center','center', [255,255,255],90); 
                elseif ExpCond(TSBlockCounter)==3;
                    DrawFormattedText(w, textGuidedPGo, 'center','center', [255,255,255],90);  
                else
                    DrawFormattedText(w, textGuidedPNoGo, 'center','center', [255,255,255],90); 
                end
        
                Screen(w,'Flip'); 
                [pressed, ~] = KeyPressFunction(space, inf); 
                
                if pressed==1  % Spacebar was pressed, do the experiment.
                
                    % loop through the experimental trials    
                    for ExpTrialNO=1:size(CurrentExpCond,1); 
                        trialNo=RandomTrials(ExpTrialNO,:); 

                        % ITI screen
                        [X,Y] = RectCenter(wRect); 
                        FixCross = [X-4,Y-30,X+4,Y+30;X-30,Y-4,X+30,Y+4]; 
                        Screen('FillRect', w, [255,255,255], FixCross'); 
                        Screen('Flip', w); 
                        [pressed, ~] = KeyPressFunction(esc, iti);
                        if pressed ==1 
                            debrief=0;
                            break 
                        end

                         %%%% Send shock code
                        if TScondition==1; % if they are in the threat condition
                            if threatBlockCount==2; % if it is the second threat block...
                                if ExpTrialNO==30; % ...and if it is trial number 30...
                                    OJR_sendshock(intensity); % ...send shock.
                                    %disp('should be trial 30') % test the code
                                end
                            elseif threatBlockCount==4; % if it is the fourth threat block...
                                if ExpTrialNO==3; ...and if it is trial 3...
                                    OJR_sendshock(intensity); % ...send shock.
                                  %disp('should be trial 3') % test the code
                                end
                            end
                        end

                        %%% Stimuli presentation
                        if trialNo(1)==1;  % if stimulus is easy and majority of tiles are yellow.
                            Screen('PutImage',w,randCheckerboard(blue,yellow,16,500,500)); 
                        elseif trialNo(1)==2;  % if stimulus is easy and majority of tiles are blue.   
                            Screen('PutImage',w,randCheckerboard(yellow,blue,16,500,500));  
                        elseif trialNo(1)==3;  % if stimulus is difficult and majority of tiles are yellow.
                            Screen('PutImage',w,randCheckerboard(blue,yellow,13,500,500));  
                        else % if stimulus is difficult and majority of tiles are blue.
                            Screen('PutImage',w, randCheckerboard(yellow,blue,13,500,500));    
                        end

                        Screen(w,'Flip'); 
                        [pressed_lr, RT] = KeyPressFunction([space, esc], StimDuration); % get participant response, OR not. 
                        WaitSecs(StimDuration-RT);   
                        if RT>=StimDuration; 
                            RT=NaN; 
                        end
                        if pressed_lr ==2 
                            debrief=0;
                            break 
                        end

                        % ITI screen
                        [X,Y] = RectCenter(wRect); 
                        FixCross = [X-4,Y-30,X+4,Y+30;X-30,Y-4,X+30,Y+4]; 
                        Screen('FillRect', w, [255,255,255], FixCross'); 
                        Screen('Flip', w);  
                        [pressedITI, RTiti] = KeyPressFunction([space,esc], iti);
                        WaitSecs(iti-RTiti);
                        if RTiti>=iti; 
                            RTiti=NaN; 
                        end
                        if pressedITI ==2 
                            debrief=0;
                            break 
                        end

                        %%%% Outcome
                        if pressed_lr == 0; 
                            outcome = trialNo(3);  
                        else  
                            outcome = trialNo(2); 
                        end

                      
                        Screen('TextSize', w, 35); 
                        if outcome ==10;  % large reward: 10 points and a happy face
                            Screen('PutImage',w, happy);
                            DrawFormattedText(w, '+10 POINTS!', 'center',550, [0 0 0],90);  
                        elseif outcome == 1; % small reward: 1 point and a happy face 
                            Screen('PutImage',w, happy);
                            DrawFormattedText(w, '+1 POINT!', 'center',550, [0 0 0],90); 
                        elseif outcome== -10 % large punishment: lost 10 points and a fearful face
                            Screen('PutImage',w, fear);
                            DrawFormattedText(w, '-10 POINTS!', 'center',550, [0 0 0],90); 
                        elseif outcome==-1 % small punishment:lost 1 point and a fearful face
                            Screen('PutImage',w, fear);
                            DrawFormattedText(w, '-1 POINT!', 'center',550, [0 0 0],90); 
                        end 

                        Screen(w,'Flip');
                        [pressed, ~] = KeyPressFunction(esc, 1); 
                        if pressed ==1 
                            debrief=0;
                            break 
                        end

                        % Accuracy recording
                        GoNoGoResponse=0; % response is incorrect when it's 0 and correct when it is 1. 
                        StimulusType=[1, 0, 1, 0]; % 1=majority is yellow, 0 = majority is blue. 
                        if targetColour==1; % if target colour is blue  
                            if StimulusType(trialNo(1))==0 && pressed_lr==1; % if majority of tiles are blue and pressed a key
                                    GoNoGoResponse=1; % store response as correct.
                            elseif StimulusType(trialNo(1))==1 && pressed_lr==0; % if majority of tiles are yellow and didn't press a key
                                    GoNoGoResponse=1; % store response as correct.
                            end
                        else % target colour is yellow. 
                           if StimulusType(trialNo(1))==0 && pressed_lr==0; % if majority of tiles are blue and did not press a key...
                                    GoNoGoResponse=1; % ...store response as correct.
                            elseif StimulusType(trialNo(1))==1 && pressed_lr==1; % if majority of tiles are yellow and did press a key...
                                    GoNoGoResponse=1; % ...store response as correct.
                           end  
                        end
                        %%%% Save the responses to outputfile
                        datafilepointer = fopen(datafilename,'a+');
                        fprintf(datafilepointer,'%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d \n',subID, ExpTrialNO, 1, TScondition, targetColour, trialNo(1), ExpCond(TSBlockCounter), trialNo(2), trialNo(3), outcome, pressed_lr, GoNoGoResponse, RT, RTiti, StimDuration); 
                        fclose(datafilepointer);     
                    end
                    
                    TSBlockCounter=TSBlockCounter+1; % update threat counter
                end
            end
        end
    end

            
% Escape was pressed, go to debrief screen.
% Debrief message - end of experiment.  
Screen(w,'FillRect',[180 180 180]); 
Screen('TextSize', w, 24);  
Screen(w,'TextFont','Helvetica'); 
Screen(w,'TextStyle',0);  
DrawFormattedText(w, 'Thank you for participating in this experiment.', 'center','center', [0 0 0],90);  
Screen(w,'Flip');  
KbWait([],3); % waits for all keys to be released and then for a keypress. 
Screen('CloseAll');
     
datestr(now)

catch 
    % "catch" closes all screens in case the program written in the try section crashes. 
    Screen('CloseAll');
    rethrow(lasterror);   
end
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
   
    